//-----------------------------------------------------------------------------
// EXT_MOD_BMA180
// Version 1.5
// 02.11.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer ein externes Modul
// mit dem IC BMA180
// 3Achs-Beschleunigungs-Sensor
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_EXT_MODULE/"
// liegen und mit #include <UB_EXT_MODULE/EXT_MOD_BMA180.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt :  Port-B (Bit4) fuer ChipSelect
//           Port-B (Bit5) fuer Interrupt-Eingang  << optional
// Benutzt : SPI = Port-B (Bit1 bis Bit3)
//-----------------------------------------------------------------------------
//
// Hinweis : das Modul muss am MK3-Board entweder an der linken oder rechten
//           Buchsenleiste angesteckt werden
//           
// Patchplan :
//
// 4 bzw 5 Leitungen muessen verdrahtet werden
// (in der Default-Einstellung liegt alles auf Port-B)
//
// Port-B1 (SPI-SCK)  muss zum BMA-Modul Pin 19  (BMA180 SCK)
// Port-B2 (SPI-MOSI) muss zum BMA-Modul Pin 18  (BMA180 SDI)
// Port-B3 (SPI-MISO) muss zum BMA-Modul Pin 17  (BMA180 SDO)
// Port-B4 (Chip-Sel) muss zum BMA-Modul Pin 16  (BMA180 CS)
// Port-B5 (PCInt5)   muss zum BMA-Modul Pin 15  (BMA180 INT)  << optional
//
//-----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------------
#include "ub_mk3_2560/MK3_2560_SPI.h" //  bindet die SPI-Funktionen ein
#include <avr/interrupt.h>


//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "EXT_INIT_BMA180" starten
// 01A. Die Variable : <EXT_BMA180_OK> zeigt an ob OK oder FEHLER
// 02. Funktion "MK3_BMA180_setFilter" zum einstellen vom Filter
// 03. Funktion "MK3_BMA180_setRange" zum einstellen der Range
// [POLLING-MODE]
// 04. Funktion "MK3_BMA180_getData" starten
// 04B. Die Beschleunigungs-Daten stehen dann in der Struktur : <EXT_BMA180>
// [INTERRUPT-MODE]
// 05. Funktion "MK3_BMA180_enableINT" zum ein/ausschalten vom Interrupt-Mode
// 05A. Die Variable : <new_data zeigt> an ob neue Daten vorhanden sind
// 05B. Die Beschleunigungs-Daten stehen dann in der Struktur : <EXT_BMA180>
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------
typedef enum { // Filter einstellungen
  BMA_FILTER_10Hz = 0,  
  BMA_FILTER_20Hz,    
  BMA_FILTER_40Hz,      
  BMA_FILTER_75Hz,
  BMA_FILTER_150Hz,
  BMA_FILTER_300Hz,
  BMA_FILTER_600Hz,
  BMA_FILTER_1200Hz,      // (default)
  BMA_FILTER_HIGH_1Hz,
  BMA_FILTER_BAND_PASS
}BMA180_FILTER; 

typedef enum { // RANGE einstellung  [z.B. 1G = +/- 1g]
  BMA_RANGE_1G =0,        // (default)
  BMA_RANGE_1G5,
  BMA_RANGE_2G,
  BMA_RANGE_3G,
  BMA_RANGE_4G,
  BMA_RANGE_8G,
  BMA_RANGE_16G
}BMA180_RANGE;



//-----------------------------------------------------------------------------
// Die gelesenen Beschleunigungs-Werte werden in einer Struktur uebergeben
// um im externen Programm keine Umwandlung machen zu mssen
//-----------------------------------------------------------------------------
typedef struct {
  BMA180_FILTER filter;       // aktuelle Filter einstellung
  BMA180_RANGE range;         // aktuelle Range einstellung
  bool int_mode;              // interrupt-Mode EIN/AUS
  unsigned int int_speed;     // interrupt-Speed
}Settings_t;

typedef struct {
  bool new_data;              // neue Daten vorhanden
  unsigned int x_achse;       // absolut-Wert der X-Achse
  bool x_negativ;	      // true wenn X-Achse negativ
  unsigned int y_achse;       // absolut-Wert der Y-Achse
  bool y_negativ;             // true wenn Y-Achse negativ
  unsigned int z_achse;       // absolut-Wert der Z-Achse
  bool z_negativ;             // true wenn Z-Achse negativ
  unsigned char temperatur;   // absolut-Wert der Temperatur
  bool t_negativ;             // true wenn Temperatur negativ
}RAW_Data_t;

typedef struct {
  float x_achse;              // Y-Achse    [in mg]
  float y_achse;              // Y-Achse    [in mg]
  float z_achse;              // Y-Achse    [in mg]
  float temperatur;           // Temperatur [in Grad_Celsius]
}Unit_Data_t;

typedef struct {
  Settings_t setting;         // einstellungen
  RAW_Data_t raw;             // ROH-Daten vom Sensor (ohne Umrechnung)
  Unit_Data_t data;           // Daten mit Einheiten
}BMA180_t;


//-----------------------------------------------------------------------------
// Globale Variabeln
//-----------------------------------------------------------------------------
bool EXT_BMA180_OK=false;       // zum ueberpruefen der fehlerfreien Funktion
BMA180_t EXT_BMA180;		// hier stehen die Istwerte drinn




//-----------------------------------------------------------------------------
// Port-Pin definition
//-----------------------------------------------------------------------------

#define MK3_DDR_BMA180 DDRB	// DDR-Register der BMA180-Signale
#define MK3_PORT_BMA180 PORTB	// Port der BMA180-Signale
#define MK3_PIN_BMA180 PINB     // Pin der BMA180-Signale (fuer INT) 

#define BMA180_CS  0x10         // ChipSelect = Bit4
#define BMA180_INT 0x20         // Interrupt  = Bit5

#define BMA180_DEFAULT_ID  0x03 // ID-Nummer des BMA180


//-----------------------------------------------------------------------------
// Register Definitionen vom BMA180 (siehe Datasheet Seite 21)
//-----------------------------------------------------------------------------

#define BMA_REG_ID     0x00   // ID-Nummer
#define BMA_REG_XLSB   0x02   // X-Achse (LSB)
#define BMA_REG_XMSB   0x03   // X-Achse (MSB)
#define BMA_REG_YLSB   0x04   // Y-Achse (LSB)
#define BMA_REG_YMSB   0x05   // Y-Achse (MSB)
#define BMA_REG_ZLSB   0x06   // Z-Achse (LSB)
#define BMA_REG_ZMSB   0x07   // Z-Achse (MSB)
#define BMA_REG_TEMP   0x08   // Temperatur
#define BMA_REG_CTRL0  0x0D   // Control-Register-0 (fuer ee_w)
#define BMA_REG_FILTER 0x20   // Analog-Filter
#define BMA_REG_CTRL3  0x21   // Control-Register-3 (fuer INT)
#define BMA_REG_RANGE  0x35   // Range Einstellung

//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void EXT_INIT_BMA180(void);     // init vom BMA180
void MK3_BMA180_getData(void);  // auslesen der Beschleunigungs-Daten
void MK3_BMA180_setFilter(BMA180_FILTER filter); // stellt den Filter ein
void MK3_BMA180_setRange(BMA180_RANGE range);    // stellt die Range ein
void MK3_BMA180_enableINT(bool mode);            // stellt Interrupt EIN/AUS


//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
char MK3_BMA180_checkID(void);
char MK3_BMA180_readByte(char adr);
void MK3_BMA180_writeByte(char adr, char wert);
void MK3_BMA180_makeUnits(void);



//-----------------------------------------------------------------------------
void EXT_INIT_BMA180(void) {	// zum initialisieren des externen BMA180

  // init der Chip-Select-Leitung
  MK3_DDR_BMA180|=BMA180_CS;    // Chip-Select als Ausgang
  MK3_PORT_BMA180|=BMA180_CS;   // Chip-Select AUS

  // init vom SPI-BUS
  MK3_INIT_SPI();

  // init der Struktur
  EXT_BMA180.setting.filter=BMA_FILTER_1200Hz;
  EXT_BMA180.setting.range=BMA_RANGE_1G;
  EXT_BMA180.setting.int_mode=false;
  EXT_BMA180.setting.int_speed=300;

  EXT_BMA180.raw.new_data=false;
  EXT_BMA180.raw.x_achse=0;
  EXT_BMA180.raw.x_negativ=false;
  EXT_BMA180.raw.y_achse=0;
  EXT_BMA180.raw.y_negativ=false;
  EXT_BMA180.raw.z_achse=0;
  EXT_BMA180.raw.z_negativ=false;
  EXT_BMA180.raw.temperatur=0;
  EXT_BMA180.raw.t_negativ=false;

  EXT_BMA180.data.x_achse=0;
  EXT_BMA180.data.y_achse=0;
  EXT_BMA180.data.z_achse=0;
  EXT_BMA180.data.temperatur=0;
  
  // test ob BMA180 ok
  if(MK3_BMA180_checkID()!=BMA180_DEFAULT_ID) {
    // BMA180 ist nicht ok
    EXT_BMA180_OK=false; 
  }
  else {
    // BMA180 ist OK
    EXT_BMA180_OK=true;
    // default Settings einstellen
    MK3_BMA180_setFilter(BMA_FILTER_1200Hz);  // Filter auf 1200Hz
    MK3_BMA180_setRange(BMA_RANGE_1G);        // Range auf +/- 1g
    MK3_BMA180_enableINT(false);              // Polling-Mode
  }
}


//-----------------------------------------------------------------------------
void MK3_BMA180_getData(void) {  // auslesen der Daten
  // auslesen der X,Y,Z-Daten (Lo-byte First)
  // und der Temperatur
  // 
  // Hier werden die RAW-Data aus dem BMA-180 gelesen
  // es sind <keine> Physikalische Einheiten hinterlegt
  //
  // Reihenfolge ist wichtig : T,X,Y,Z

  unsigned char lo,hi;
  unsigned int wert;
  unsigned int dummy;

  //---------------------
  // Temperatur auslesen
  //---------------------

  // Byte = Bit7 bis Bit0
  lo=MK3_BMA180_readByte(BMA_REG_TEMP);

  // test ob positiv oder negativ
  if((lo&0x80)!=0) {
    // negativ
    hi=0xFF^lo;
    hi++;  // zweierkomplement
    lo=hi;
    EXT_BMA180.raw.t_negativ=true;
  }
  else {
    // positiv
    EXT_BMA180.raw.t_negativ=false;
  }

  // Temperatur speichern
  EXT_BMA180.raw.temperatur=lo;

  //---------------------
  // X-Achse auslesen
  //---------------------

  // Lo-byte = Bit7 bis Bit2
  lo=MK3_BMA180_readByte(BMA_REG_XLSB);
  lo=(lo>>2);
  lo=(lo&0x3F);

  // Hi-Byte = Bit7 bis Bit0
  hi=MK3_BMA180_readByte(BMA_REG_XMSB);
  
  // 14bit Wert erstellen
  wert=(hi*64)+lo;

  // test ob positiv oder negativ
  if((wert&0x2000)!=0) {
    // negativ
    dummy=0x3FFF^wert;
    dummy++;  // zweierkomplement
    wert=dummy;
    EXT_BMA180.raw.x_negativ=true;
  }
  else {
    // positiv
    EXT_BMA180.raw.x_negativ=false;
  }

  // X-Wert speichern
  EXT_BMA180.raw.x_achse=wert;

  //---------------------
  // Y-Achse auslesen
  //---------------------

  // Lo-byte = Bit7 bis Bit2
  lo=MK3_BMA180_readByte(BMA_REG_YLSB);
  lo=(lo>>2);
  lo=(lo&0x3F);

  // Hi-Byte = Bit7 bis Bit0
  hi=MK3_BMA180_readByte(BMA_REG_YMSB);
  
  // 14bit Wert erstellen
  wert=(hi*64)+lo;

  // test ob positiv oder negativ
  if((wert&0x2000)!=0) {
    // negativ
    dummy=0x3FFF^wert;
    dummy++;  // zweierkomplement
    wert=dummy;
    EXT_BMA180.raw.y_negativ=true;
  }
  else {
    // positiv
    EXT_BMA180.raw.y_negativ=false;
  }

  // Y-Wert speichern
  EXT_BMA180.raw.y_achse=wert;

  //---------------------
  // Z-Achse auslesen
  //---------------------

  // Lo-byte = Bit7 bis Bit2
  lo=MK3_BMA180_readByte(BMA_REG_ZLSB);
  lo=(lo>>2);
  lo=(lo&0x3F);

  // Hi-Byte = Bit7 bis Bit0
  hi=MK3_BMA180_readByte(BMA_REG_ZMSB);
  
  // 14bit Wert erstellen
  wert=(hi*64)+lo;

  // test ob positiv oder negativ
  if((wert&0x2000)!=0) {
    // negativ
    dummy=0x3FFF^wert;
    dummy++;  // zweierkomplement
    wert=dummy;
    EXT_BMA180.raw.z_negativ=true;
  }
  else {
    // positiv
    EXT_BMA180.raw.z_negativ=false;
  }

  // Z-Wert speichern
  EXT_BMA180.raw.z_achse=wert;

  // Umrechnen der Roh-Daten
  MK3_BMA180_makeUnits();
}

//-----------------------------------------------------------------------------
void MK3_BMA180_setFilter(BMA180_FILTER filter) { // stellt den Filter ein
  unsigned char wert=0x00,dummy;

  // damit ein Wert im EEprom vom BMA180 geaendert werden kann
  // muss zuerst das Bit <EE_W> im Register <0x0D> gesetzt werden

  // zuerst den alten Wert auslesen
  dummy=MK3_BMA180_readByte(BMA_REG_CTRL0);
  dummy|=0x10; // Bit <EE_W> setzen
  MK3_BMA180_writeByte(BMA_REG_CTRL0,dummy);


  // Filter einstellen
  if(filter==BMA_FILTER_10Hz) wert=0x00;
  if(filter==BMA_FILTER_20Hz) wert=0x10;
  if(filter==BMA_FILTER_40Hz) wert=0x20;
  if(filter==BMA_FILTER_75Hz) wert=0x30;
  if(filter==BMA_FILTER_150Hz) wert=0x40;
  if(filter==BMA_FILTER_300Hz) wert=0x50;
  if(filter==BMA_FILTER_600Hz) wert=0x60;
  if(filter==BMA_FILTER_1200Hz) wert=0x70;
  if(filter==BMA_FILTER_HIGH_1Hz) wert=0x80;
  if(filter==BMA_FILTER_BAND_PASS) wert=0x90;

  // zuerst den alten Wert auslesen
  dummy=MK3_BMA180_readByte(BMA_REG_FILTER);
  dummy&=0x0F;  // alten Filter-Wert ausblenden
  dummy|=wert;  // neuen wert hinzufuegen
  // neuen Wert schreiben
  MK3_BMA180_writeByte(BMA_REG_FILTER,dummy);

  // Struktur setzen
  EXT_BMA180.setting.filter=filter; 
}


//-----------------------------------------------------------------------------
void MK3_BMA180_setRange(BMA180_RANGE range) {  // stellt die Range ein
  unsigned char wert=0x00,dummy;

  // damit ein Wert im EEprom vom BMA180 geaendert werden kann
  // muss zuerst das Bit <EE_W> im Register <0x0D> gesetzt werden

  // zuerst den alten Wert auslesen
  dummy=MK3_BMA180_readByte(BMA_REG_CTRL0);
  dummy|=0x10; // Bit <EE_W> setzen
  MK3_BMA180_writeByte(BMA_REG_CTRL0,dummy);

  // Range einstellen
  if(range==BMA_RANGE_1G) wert=0x00;
  if(range==BMA_RANGE_1G5) wert=0x02;
  if(range==BMA_RANGE_2G) wert=0x04;
  if(range==BMA_RANGE_3G) wert=0x06;
  if(range==BMA_RANGE_4G) wert=0x08;
  if(range==BMA_RANGE_8G) wert=0x0A;
  if(range==BMA_RANGE_16G) wert=0x0C;

  // zuerst den alten Wert auslesen
  dummy=MK3_BMA180_readByte(BMA_REG_RANGE);
  dummy&=0xF1;  // alten Range-Wert ausblenden
  dummy|=wert;  // neuen wert hinzufuegen
  // neuen Wert schreiben
  MK3_BMA180_writeByte(BMA_REG_RANGE,dummy);

  // Struktur setzen
  EXT_BMA180.setting.range=range; 
}


//-----------------------------------------------------------------------------
void MK3_BMA180_enableINT(bool mode) {            // stellt Interrupt EIN/AUS
  unsigned char wert=0x00,dummy;

  if(mode==true) {
    // interrupt-Mode einschalten

    
    // init der Interrupt-Leitung
    MK3_DDR_BMA180&=~BMA180_INT;  // Interrupt-Pin als Eingang
    MK3_PORT_BMA180|=BMA180_INT;  // Pull-Up aktivieren

    // init der Register
    PCICR|=0x01;                  // PCINT 0-7 enable
    PCMSK0|=0x20;                 // PCINT5 enable

    // register-Wert (new_data_int=1)
    wert=0x02;

    // Global-Interrupt
    SREG|=0x80; // Alle Interrupts einschalten

    // Struktur setzen
    EXT_BMA180.setting.int_mode=true;
  }
  else {
    // interrupt-Mode ausschalten


    // init der Interrupt-Leitung
    MK3_DDR_BMA180&=~BMA180_INT;  // Interrupt-Pin als Eingang
    MK3_PORT_BMA180&=~BMA180_INT; // Pull-Up deaktivieren

    // init der Register
    PCICR&=~0x01;                 // PCINT 0-7 disable
    PCMSK0&=~0x20;                // PCINT5 disable

    // register-Wert (new_data_int=0)
    wert=0x00;

    // Struktur setzen
    EXT_BMA180.setting.int_mode=false;
  }

  // damit ein Wert im EEprom vom BMA180 geaendert werden kann
  // muss zuerst das Bit <EE_W> im Register <0x0D> gesetzt werden

  // zuerst den alten Wert auslesen
  dummy=MK3_BMA180_readByte(BMA_REG_CTRL0);
  dummy|=0x10; // Bit <EE_W> setzen
  MK3_BMA180_writeByte(BMA_REG_CTRL0,dummy);

  // zuerst den alten Wert auslesen
  dummy=MK3_BMA180_readByte(BMA_REG_CTRL3);
  dummy&=0xFD;  // alten Interrupt-Wert ausblenden
  dummy|=wert;  // neuen wert hinzufuegen
  // neuen Wert schreiben
  MK3_BMA180_writeByte(BMA_REG_CTRL3,dummy);
}


//-----------------------------------------------------------------------------
char MK3_BMA180_checkID(void) {  // INTERNE-Funktion
  // auslesen der ID-Nummer
  // Fixer-Wert : 0x03 = 03d
  char ret_wert=0;


  ret_wert=MK3_BMA180_readByte(BMA_REG_ID);  
}

//-----------------------------------------------------------------------------
char MK3_BMA180_readByte(char adr) { // INTERNE-Funktion
  // sendet und empfaengt ein Byte vom BMA180
  char ret_wert=0;
  
  adr|=0x80;                   // Mode=Read

  MK3_PORT_BMA180&=~BMA180_CS;  // Chip-Select EIN

  // zuerst die Adresse senden
  MK3_SPI_SEND(adr);

  // dann den Inhalt auslesen
  ret_wert=MK3_SPI_SEND(0x55);  // 0x55 = dummy kommando
  

  MK3_PORT_BMA180|=BMA180_CS;   // Chip-Select AUS

  return (ret_wert);
}


//-----------------------------------------------------------------------------
void MK3_BMA180_writeByte(char adr, char wert) { // INTERNE-Funktion
  // schreibt ein Byte in den BMA180
  unsigned int n;
  
  adr&=~0x80;                   // Mode=Write

  MK3_PORT_BMA180&=~BMA180_CS;  // Chip-Select EIN

  // zuerst die Adresse senden
  MK3_SPI_SEND(adr);

  
  for(n=0;n<5000;n++) {
    // kleine Pause (WICHTIG !!)
  }

  // dann die Daten senden
  MK3_SPI_SEND(wert);

  for(n=0;n<5000;n++) {
    // kleine Pause (WICHTIG !!)
  }
  

  MK3_PORT_BMA180|=BMA180_CS;   // Chip-Select AUS
}

//-----------------------------------------------------------------------------
void MK3_BMA180_makeUnits(void)  { // INTERNE-Funktion
  // rechnet die Roh-Daten in Einheiten um

  float lsb1=0,wert=0;
  
  //---------------------------
  // Temperatur umrechnen
  //---------------------------

  if(EXT_BMA180.raw.t_negativ==false) {
    // wenn positiv
    lsb1=0.5;                                    // 1LSB = 0,5C
    wert=(lsb1*EXT_BMA180.raw.temperatur)+24.0;  // Temp bei 0 Digit = 24,0C
    EXT_BMA180.data.temperatur=wert;
  }
  else {
    // wenn negativ
    lsb1=0.5;                                    // 1LSB = 0,5C
    wert=24.0-(lsb1*EXT_BMA180.raw.temperatur);  // Temp bei 0 Digit = 24,0C
    EXT_BMA180.data.temperatur=wert;    
  }

  // Faktor fuer X,Y,Z-Achse
  if(EXT_BMA180.setting.range==BMA_RANGE_1G) lsb1=(2000.0/16383);  // +/- 1g in 14 bit
  if(EXT_BMA180.setting.range==BMA_RANGE_1G5) lsb1=(3000.0/16383);  // +/- 1.5g in 14 bit
  if(EXT_BMA180.setting.range==BMA_RANGE_2G) lsb1=(4000.0/16383);  // +/- 2g in 14 bit
  if(EXT_BMA180.setting.range==BMA_RANGE_3G) lsb1=(6000.0/16383);  // +/- 3g in 14 bit
  if(EXT_BMA180.setting.range==BMA_RANGE_4G) lsb1=(8000.0/16383);  // +/- 4g in 14 bit
  if(EXT_BMA180.setting.range==BMA_RANGE_8G) lsb1=(16000.0/16383);  // +/- 8g in 14 bit
  if(EXT_BMA180.setting.range==BMA_RANGE_16G) lsb1=(32000.0/16383);  // +/- 16g in 14 bit

  //---------------------------
  // X-Achse umrechnen
  //---------------------------
  if(EXT_BMA180.raw.x_negativ==false) {
    // wenn positiv
    wert=(lsb1*EXT_BMA180.raw.x_achse);
    EXT_BMA180.data.x_achse=wert;
  }
  else {
    // wenn negativ
    wert=0.0-(lsb1*EXT_BMA180.raw.x_achse);
    EXT_BMA180.data.x_achse=wert;
  }

  //---------------------------
  // Y-Achse umrechnen
  //---------------------------
  if(EXT_BMA180.raw.y_negativ==false) {
    // wenn positiv
    wert=(lsb1*EXT_BMA180.raw.y_achse);
    EXT_BMA180.data.y_achse=wert;
  }
  else {
    // wenn negativ
    wert=0.0-(lsb1*EXT_BMA180.raw.y_achse);
    EXT_BMA180.data.y_achse=wert;
  }

  //---------------------------
  // Z-Achse umrechnen
  //---------------------------
  if(EXT_BMA180.raw.z_negativ==false) {
    // wenn positiv
    wert=(lsb1*EXT_BMA180.raw.z_achse);
    EXT_BMA180.data.z_achse=wert;
  }
  else {
    // wenn negativ
    wert=0.0-(lsb1*EXT_BMA180.raw.z_achse);
    EXT_BMA180.data.z_achse=wert;
  }
}


//-----------------------------------------------------------------------------
ISR (PCINT0_vect) { // ISR : PinChange 0-7 Interrupt   // INTERNE-Funktion
  // der Interrupt wird ausgeloest,
  // wenn neue Daten vom BMA180 verfuegbar sind
  //
  // weil die Daten so schnell vom BMA180 aktuallisiert werden,
  // wird das ganze um den Wert <int_speed> verlangsamt
  //

  unsigned char wert,dummy;
  static unsigned int bma_speed=0;
 
  if(EXT_BMA180.setting.int_mode==true) {
    // wenn Interrupt-Mode aktiv
    wert=MK3_PIN_BMA180;
    if((wert&BMA180_INT)!=0) {
      // bei einem High-Pegel vom PCINT5-Pin5
      bma_speed++;
      if(bma_speed>=EXT_BMA180.setting.int_speed) {
        bma_speed=0;

        // aktuelle Sensor-Daten auslesen
        MK3_BMA180_getData();
        // merker fuer neue Daten setzen
        EXT_BMA180.raw.new_data=true;
      }
      else {
        // dummy auslesen, damit das INT-Flag geloescht wird
        dummy=MK3_BMA180_readByte(BMA_REG_XLSB);
        dummy=MK3_BMA180_readByte(BMA_REG_XMSB);       
      }
    }
  }
}



